//--------------------------------------------------------------------------
//--------------------------------------------------------------------------
// TITLE : High Performance counter unit - HPTimer.pas
//
// DESCRIPTION : Timing routines based on the Windows high perf. counter.
//
// DATE : 21/10/13
//
// MODIFICATIONS :
//    04/10/11 Created
//    21/10/13 Modified HPWait function period param. to ms
//
// AUTHOR : Martyn Brown
//
//  2011-13 Netronics Ltd. All rights reserved.
//--------------------------------------------------------------------------
//--------------------------------------------------------------------------
unit HPTimer;

interface

uses
  Windows;

function HPWait(Period : Integer) : Boolean;

implementation

//--------------------------------------------------------------------------
// HPWait
//
// Wait for a period of time.
//
// Note : The period is specified in milliseconds & is measured using the
// high performance counter.
//
// Returns
//    False = Error
//    True = OK
//--------------------------------------------------------------------------
function HPWait(Period : Integer) : Boolean;
const
  MILLISECOND : Integer = 1000;
var
  CountsPerSecond, PeriodCounts, StartCount, CurrentCount, TerminalCount : Int64;
begin
  if QueryPerformanceFrequency(CountsPerSecond) then
  begin
    // High perf. counter avail., so pause execution for specified period
    if QueryPerformanceCounter(StartCount) then
    begin
      Result := True;
      PeriodCounts := (CountsPerSecond * Period) div MILLISECOND;
      TerminalCount := StartCount + PeriodCounts;
      repeat
        // Loop until period specified elapsed
        if not QueryPerformanceCounter(CurrentCount) then
          Result := False;  // Error
      until not Result or (CurrentCount > TerminalCount);
    end
    else
      Result := False;  // Error
  end
  else
    // High perf. counter not avail.
    Result := False;  // Error
end;
//--------------------------------------------------------------------------
end.
//--------------------------------------------------------------------------
//--------------------------------------------------------------------------
