//--------------------------------------------------------------------------
//--------------------------------------------------------------------------
// TITLE : CANdoTransferTest main routines - Main.pas
//
// DESCRIPTION : Delphi program to test the transfer of CAN frames from
// one CANdo to another. Two copies of this program are required, each conn-
// ected to a CANdo device, one set to transmit & the other to receive.
//
// TRANSMIT
// The transmit CANdo, sends a stream of CAN frames, each containing 8 bytes
// of data, spaced in time by the Interframe Gap, to the receive CANdo. Each
// frame contains a frame count in the first 4 bytes & a CRC checksum for
// the frame in the last 2 bytes.
//
// RECEIVE
// The receive CANdo, receives the transmitted frames on the CAN bus & then
// the program checks the frame count to make sure that the frames arrive in
// the correct order. The program also compares the received checksum in
// each frame with a calculated checksum for that frame. These two checks
// combined prove that the the frames are in the correct order & that each
// byte within each frame is received correctly.
//
// DATE : 21/10/13
//
// MODIFICATIONS :
//    11/07/11 Created
//    02/10/11 Frame count & checksum added to each frame
//    04/10/11 CAN transmit moved to separate thread & timed using the
//             high perf. timer
//    21/10/13 1) Updated for CANdo API v4.0
//             2) CANdo AUTO support added
//
// AUTHOR : Martyn Brown
//
//  2011-13 Netronics Ltd. All rights reserved.
//--------------------------------------------------------------------------
//--------------------------------------------------------------------------
unit Main;

interface

uses
  Windows, Messages, SysUtils, DateUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, Spin, ComCtrls, CRC, CANdoImport, XPMan;

  const
  IDS = $18F00020;  // ID used for start frame
  IDD = $18F00030;  // ID used for data frames

type
  TForm1 = class(TForm)
    Panel1: TPanel;
    Label1: TLabel;
    ComboBox1: TComboBox;
    StatusBar1: TStatusBar;
    Panel2: TPanel;
    RadioButton3: TRadioButton;
    RadioButton4: TRadioButton;
    Label5: TLabel;
    Gap: TSpinEdit;
    Label6: TLabel;
    Button1: TButton;
    Button2: TButton;
    ProgressBar1: TProgressBar;
    Edit2: TEdit;
    Label4: TLabel;
    Button3: TButton;
    Timer1: TTimer;
    Label7: TLabel;
    Label3: TLabel;
    NoOfFrames: TSpinEdit;
    XPManifest1: TXPManifest;
    procedure RadioButton4Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
  private
    { Private declarations }
    CANdoAPIDLLFoundFlag, RunState : Boolean;
    StartTime, EndTime : TDateTime;
    TransmitThread : TThread;
    procedure Wait(Delay : Longint);
    procedure GetStatus;
    procedure Transmit;
    procedure TransmitStop;
    procedure TransmitThreadTerminating(Sender: TObject);
    procedure Receive;
  public
    { Public declarations }
    TransmitRunningFlag : Boolean;
    FrameCount, TotalFrames : Integer;
    XSum : Word;
    CANdoUSB : TCANdoUSB;  // Store for parameters relating to connected CANdo
  end;

var
  Form1: TForm1;

implementation

uses
  CANTransmit;

var
  CANdoCANBuffer : TCANdoCANBuffer;  // Cyclic store for CAN messages collected from CANdo
  CANdoStatus : TCANdoStatus;  // Store for status message collected from CANdo

{$R *.dfm}

//--------------------------------------------------------------------------
procedure TForm1.FormCreate(Sender: TObject);
var
  CANdoDevice : TCANdoDevice;
  Status : Integer;
begin
  // Dynamically load CANdo.dll
  if CANdoInitialise then
  begin
    CANdoAPIDLLFoundFlag := True;
    CANdoDevice.HardwareType := CANDO_TYPE_ANY;
    CANdoDevice.SerialNo := '';
    Status := CANdoOpenDevice(@CANdoUSB, @CANdoDevice);
    if Status = CANDO_SUCCESS then
    begin
      StatusBar1.Panels[0].Text := String(CANdoUSB.Description)+' S/N '+String(CANdoUSB.SerialNo);
      GetStatus;
      // Set min. inter-frame gap for transmitter dependent upon device type connected
      if CANdoDevice.HardwareType = CANDO_TYPE_CANDO then
      begin
        // 2ms min. inter-frame gap for CANdo
        Gap.MinValue := 2;
        Gap.Value := 2;
      end;
    end
    else
    if Status = CANDO_USB_DLL_ERROR then
      StatusBar1.Panels[0].Text := 'CANdo USB DLL not found'
    else
    if Status = CANDO_USB_DRIVER_ERROR then
      StatusBar1.Panels[0].Text := 'CANdo driver not found'
    else
      StatusBar1.Panels[0].Text := 'CANdo not found';
  end
  else
  begin
    StatusBar1.Panels[0].Text := 'CANdo API DLL not found';
    CANdoAPIDLLFoundFlag := False;
  end;
  RunState := False;  // Stopped
end;
//--------------------------------------------------------------------------
procedure TForm1.GetStatus;
begin
  if CANdoUSB.OpenFlag then
  begin
    if CANdoRequestStatus(@CANdoUSB) = CANDO_SUCCESS then
    begin
      if not Timer1.Enabled then
      begin
        Wait(10);  // Wait 10ms for a reply from CANdo
        if CANdoReceive(@CANdoUSB, @CANdoCANBuffer, @CANdoStatus) = CANDO_SUCCESS then
        begin
          if CANdoStatus.NewFlag = CANDO_DEVICE_STATUS then
          begin
            // New status message received
            StatusBar1.Panels[1].Text := 'H/W v'+Format('%.1f', [CANdoStatus.HardwareVersion / 10.0])+
              '   S/W v'+Format('%.1f', [CANdoStatus.SoftwareVersion / 10.0])+'   Status='+IntToHex(CANdoStatus.Status, 2)+
              '   Bus State='+IntToHex(CANdoStatus.BusState, 2)+'   Timestamp='+IntToStr(CANdoStatus.TimeStamp);
            CANdoStatus.NewFlag := CANDO_NO_STATUS;  // Clear flag to indicate status read
          end
          else
            StatusBar1.Panels[1].Text := 'Status unknown';
        end
        else
          StatusBar1.Panels[1].Text := 'Error communicating with CANdo';
      end;
    end
    else
      StatusBar1.Panels[1].Text := 'Error communicating with CANdo';
  end;
end;
//--------------------------------------------------------------------------
procedure TForm1.Button1Click(Sender: TObject);
var
  Status : Integer;
begin
  // Start button
  if CANdoUSB.OpenFlag and not RunState then
  begin
    // Display device status
    StatusBar1.Panels[0].Text := String(CANdoUSB.Description)+' S/N '+String(CANdoUSB.SerialNo);
    GetStatus;
    // Set baud rate
    case ComboBox1.ItemIndex of
      0: Status := CANdoSetBaudRate(@CANdoUSB, 0, 3, 7, 7, 2, 0);  // 125k
      1: Status := CANdoSetBaudRate(@CANdoUSB, 0, 1, 7, 7, 2, 0);  // 250k
      3: Status := CANdoSetBaudRate(@CANdoUSB, 0, 0, 2, 2, 2, 0);  // 1M
    else
      Status := CANdoSetBaudRate(@CANdoUSB, 0, 0, 7, 7, 2, 0);  // 500k
    end;
    if Status = CANDO_SUCCESS then
    begin
      Wait(100);  // Wait 100ms to allow CANdo to store baud rate in EEPROM, in case modified
      // Set mode to 'Normal'
      if CANdoSetMode(@CANdoUSB, CANDO_NORMAL_MODE) = CANDO_SUCCESS then
      begin
        Wait(10);  // Wait 10ms to allow CANdo to store mode in EEPROM, in case modified
        // Set filters to accept all messages
        if CANdoSetFilters(@CANdoUSB,
          0,
          CANDO_ID_29_BIT, 0,
          CANDO_ID_11_BIT, 0,
          0,
          CANDO_ID_29_BIT, 0,
          CANDO_ID_11_BIT, 0,
          CANDO_ID_29_BIT, 0,
          CANDO_ID_11_BIT, 0) = CANDO_SUCCESS then
          begin
            Wait(10);  // Wait 10ms to allow filters to be configured in CAN module
            // Flush USB buffers
            if CANdoFlushBuffers(@CANdoUSB) = CANDO_SUCCESS then
              // Set CANdo state to run
              if CANdoSetState(@CANdoUSB, CANDO_RUN) = CANDO_SUCCESS then
                RunState := True;  // Running
          end;
      end;
    end;
  end;

  if RunState then
  begin
    // CANdo running
    if RadioButton3.Checked then
      Transmit // CANdo transmit
    else
      Receive;  // CANdo receive
    RadioButton3.Enabled := False;
    RadioButton4.Enabled := False;
    Button1.Enabled := False;
    Button2.Enabled := True;
    Edit2.Text := '0';
    Edit2.Color := clWindow;
  end;
end;
//--------------------------------------------------------------------------
procedure TForm1.Button2Click(Sender: TObject);
begin
  // Stop button
  if RadioButton3.Checked then
    TransmitStop
  else
    Timer1.Enabled := False;

  ProgressBar1.Position := 0;
  Button1.Enabled := True;
  Button2.Enabled := False;
  Label7.Caption := '';

  if CANdoUSB.OpenFlag then
  begin
    if CANdoSetState(@CANdoUSB, CANDO_STOP) = CANDO_SUCCESS then
      RunState := False  // Stopped
    else
      Label7.Caption := 'Unable to stop CANdo';
  end;
end;
//--------------------------------------------------------------------------
procedure TForm1.Button3Click(Sender: TObject);
begin
  // Close button
  Close;
end;
//--------------------------------------------------------------------------
procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  if RadioButton3.Checked then
    TransmitStop
  else
    Timer1.Enabled := False;

  if CANdoUSB.OpenFlag then
  begin
    CANdoSetState(@CANdoUSB, CANDO_STOP);
    CANdoClose(@CANdoUSB);
  end;
  CANdoFinalise;  // Unload CANdo.dll
end;
//--------------------------------------------------------------------------
// Wait
//
// Wait for Delay ms before proceeding.
//
// Returns
//    Nothing
//--------------------------------------------------------------------------
procedure TForm1.Wait(Delay : Longint);
var
	ElapsedTime, CurrentTime : Longint;
begin
  ElapsedTime := GetTickCount;
  repeat
    Application.ProcessMessages;
    CurrentTime := GetTickCount;
  until Application.Terminated or ((CurrentTime - ElapsedTime) > Delay);
end;
//--------------------------------------------------------------------------
procedure TForm1.RadioButton4Click(Sender: TObject);
begin
  if RadioButton3.Checked then
  begin
    // TRANSMIT
    NoOfFrames.Visible := True;
    Gap.Visible := True;
    Label3.Visible := True;
    Label5.Visible := True;
    Label6.Visible := True;
    ProgressBar1.Visible := False;
    Panel2.Color := clMoneyGreen;
  end
  else
  begin
    // RECEIVE
    NoOfFrames.Visible := False;
    Gap.Visible := False;
    Label3.Visible := False;
    Label5.Visible := False;
    Label6.Visible := False;
    ProgressBar1.Visible := True;
    Panel2.Color := clSilver;
  end;
  Gap.Invalidate;
end;
//--------------------------------------------------------------------------
procedure TForm1.Timer1Timer(Sender: TObject);
var
  Status, EmbeddedXSum, EmbeddedFrameCount : Integer;
begin
  // RECEIVE
  // Collect messages sent by transmitting CANdo
  Status := CANdoReceive(@CANdoUSB, @CANdoCANBuffer, @CANdoStatus);
  if Status <> CANDO_SUCCESS then
    StatusBar1.Panels[1].Text := 'Rx Error - Frame No.='+IntToStr(FrameCount)+'  Status='+IntToStr(CANdoStatus.Status)+'  Bus State='+IntToStr(CANdoStatus.BusState)+'  Function Status='+IntToStr(Status);

  // Check for CAN messages received
  while (CANdoCANBuffer.ReadIndex <> CANdoCANBuffer.WriteIndex) or CANdoCANBuffer.FullFlag do
  begin
    // Decode frame ID
    if CANdoCANBuffer.CANMessage[CANdoCANBuffer.ReadIndex].ID = IDS then
    begin
      // Start frame, so collect tot. no. of frames
      TotalFrames := CANdoCANBuffer.CANMessage[CANdoCANBuffer.ReadIndex].Data[0] +
        CANdoCANBuffer.CANMessage[CANdoCANBuffer.ReadIndex].Data[1] * 256 +
        CANdoCANBuffer.CANMessage[CANdoCANBuffer.ReadIndex].Data[2] * 65536 +
        CANdoCANBuffer.CANMessage[CANdoCANBuffer.ReadIndex].Data[3] * 16777216;
      ProgressBar1.Max := TotalFrames;
      Label7.Caption := 'Receiving '+IntToStr(TotalFrames)+' frames...';
      // Check checkbytes
      if (CANdoCANBuffer.CANMessage[CANdoCANBuffer.ReadIndex].Data[4] <> $55) and
        (CANdoCANBuffer.CANMessage[CANdoCANBuffer.ReadIndex].Data[5] <> $AA) then
      begin
        // Stop reception & indicate checkbyte error
        if CANdoSetState(@CANdoUSB, CANDO_STOP) = CANDO_SUCCESS then
          RunState := False;  // Stopped
        Timer1.Enabled := False;
        Button1.Enabled := True;
        Button2.Enabled := False;
        RadioButton3.Enabled := True;
        RadioButton4.Enabled := True;
        Label7.Caption := 'Checkbyte error in frame '+IntToStr(FrameCount)+
          '. Checkbyte 1 = '+IntToStr(CANdoCANBuffer.CANMessage[CANdoCANBuffer.ReadIndex].Data[4])+
          '. Checkbyte 2 = '+IntToStr(CANdoCANBuffer.CANMessage[CANdoCANBuffer.ReadIndex].Data[5]);
        Edit2.Color := clRed;
      end
      else
      begin
        // Check checksum
        CalculateCRC(@CANdoCANBuffer.CANMessage[CANdoCANBuffer.ReadIndex].Data, 6, XSum);
        EmbeddedXSum := CANdoCANBuffer.CANMessage[CANdoCANBuffer.ReadIndex].Data[6] +
          CANdoCANBuffer.CANMessage[CANdoCANBuffer.ReadIndex].Data[7] * 256;
        if XSum <> EmbeddedXSum then
        begin
          // Stop reception & indicate checksum error
          if CANdoSetState(@CANdoUSB, CANDO_STOP) = CANDO_SUCCESS then
            RunState := False;  // Stopped
          Timer1.Enabled := False;
          Button1.Enabled := True;
          Button2.Enabled := False;
          RadioButton3.Enabled := True;
          RadioButton4.Enabled := True;
          Label7.Caption := 'Checksum error in frame '+IntToStr(FrameCount)+'. Received checksum = '+IntToStr(EmbeddedXSum)+' Calculated checksum = '+IntToStr(XSum);
          Edit2.Color := clRed;
        end;
      end;
    end
    else
    if CANdoCANBuffer.CANMessage[CANdoCANBuffer.ReadIndex].ID = IDD then
    begin
      // Data frame, so collect frame count
      EmbeddedFrameCount := CANdoCANBuffer.CANMessage[CANdoCANBuffer.ReadIndex].Data[0] +
        CANdoCANBuffer.CANMessage[CANdoCANBuffer.ReadIndex].Data[1] * 256 +
        CANdoCANBuffer.CANMessage[CANdoCANBuffer.ReadIndex].Data[2] * 65536 +
        CANdoCANBuffer.CANMessage[CANdoCANBuffer.ReadIndex].Data[3] * 16777216;
      // Check frame count
      if FrameCount <> EmbeddedFrameCount then
      begin
        // Stop reception & indicate frame count error
        if CANdoSetState(@CANdoUSB, CANDO_STOP) = CANDO_SUCCESS then
          RunState := False;  // Stopped
        Timer1.Enabled := False;
        Button1.Enabled := True;
        Button2.Enabled := False;
        RadioButton3.Enabled := True;
        RadioButton4.Enabled := True;
        Label7.Caption := 'Frame count error in frame '+IntToStr(FrameCount)+
          '. Frame count = '+IntToStr(FrameCount)+
          '. Transmitted frame count = '+IntToStr(EmbeddedFrameCount);
        Edit2.Color := clRed;
      end
      else
      // Check checkbytes
      if (CANdoCANBuffer.CANMessage[CANdoCANBuffer.ReadIndex].Data[4] <> $55) and
        (CANdoCANBuffer.CANMessage[CANdoCANBuffer.ReadIndex].Data[5] <> $AA) then
      begin
        // Stop reception & indicate checkbyte error
        if CANdoSetState(@CANdoUSB, CANDO_STOP) = CANDO_SUCCESS then
          RunState := False;  // Stopped
        Timer1.Enabled := False;
        Button1.Enabled := True;
        Button2.Enabled := False;
        RadioButton3.Enabled := True;
        RadioButton4.Enabled := True;
        Label7.Caption := 'Checkbyte error in frame '+IntToStr(FrameCount)+
          '. Checkbyte 1 ($55) = '+IntToStr(CANdoCANBuffer.CANMessage[CANdoCANBuffer.ReadIndex].Data[4])+
          '. Checkbyte 2 ($AA) = '+IntToStr(CANdoCANBuffer.CANMessage[CANdoCANBuffer.ReadIndex].Data[5]);
        Edit2.Color := clRed;
      end
      else
      begin
        // Check checksum
        CalculateCRC(@CANdoCANBuffer.CANMessage[CANdoCANBuffer.ReadIndex].Data, 6, XSum);
        EmbeddedXSum := CANdoCANBuffer.CANMessage[CANdoCANBuffer.ReadIndex].Data[6] +
          CANdoCANBuffer.CANMessage[CANdoCANBuffer.ReadIndex].Data[7] * 256;
        if XSum <> EmbeddedXSum then
        begin
          // Stop reception & indicate checksum error
          if CANdoSetState(@CANdoUSB, CANDO_STOP) = CANDO_SUCCESS then
            RunState := False;  // Stopped
          Timer1.Enabled := False;
          Button1.Enabled := True;
          Button2.Enabled := False;
          RadioButton3.Enabled := True;
          RadioButton4.Enabled := True;
          Label7.Caption := 'Checksum error in frame '+IntToStr(FrameCount)+
            '. Received checksum = '+IntToStr(EmbeddedXSum)+' Calculated checksum = '+IntToStr(XSum);
          Edit2.Color := clRed;
        end
        else
        if FrameCount = (TotalFrames - 1) then
        begin
          // All frames received, so indicate transfer complete
          if CANdoSetState(@CANdoUSB, CANDO_STOP) = CANDO_SUCCESS then
            RunState := False;  // Stopped
          Timer1.Enabled := False;
          EndTime := Time;
          RadioButton3.Enabled := True;
          RadioButton4.Enabled := True;
          Button1.Enabled := True;
          Button2.Enabled := False;
          Edit2.Text := IntToStr(XSum);
          Label7.Caption := 'Transfer complete in '+TimeToStr(EndTime - StartTime)+'s - '+IntToStr(TotalFrames)+' frames received';
          Edit2.Color := clLime;
        end
        else
        if (FrameCount <> 0) and ((FrameCount mod (TotalFrames div 100)) = 0) then
          // Display no. of frames received
          Label7.Caption := 'Received '+IntToStr(FrameCount)+' frames of '+IntToStr(TotalFrames)+' frames...';
        Inc(FrameCount);
        ProgressBar1.Position := FrameCount;
      end;
    end;

    // Check to see if a new status message sent by CANdo
    if CANdoStatus.NewFlag = CANDO_DEVICE_STATUS then
    begin
      // New status message received
      StatusBar1.Panels[1].Text := 'H/W v'+Format('%.1f', [CANdoStatus.HardwareVersion / 10.0])+
        '   S/W v'+Format('%.1f', [CANdoStatus.SoftwareVersion / 10.0])+'   Status='+IntToHex(CANdoStatus.Status, 2)+
        '   Bus State='+IntToHex(CANdoStatus.BusState, 2)+'   Timestamp='+IntToStr(CANdoStatus.TimeStamp);
      CANdoStatus.NewFlag := CANDO_NO_STATUS;  // Clear flag to indicate status read
    end;
    // Move read pointer onto next slot in cyclic buffer
    if (CANdoCANBuffer.ReadIndex + 1) < CANDO_CAN_BUFFER_LENGTH then
      Inc(CANdoCANBuffer.ReadIndex)  // Increment index onto next free slot
    else
      CANdoCANBuffer.ReadIndex := 0;  // Wrap back to start
    CANdoCANBuffer.FullFlag := False;  // Clear flag as buffer is no longer full
  end;
end;
//--------------------------------------------------------------------------
// Transmit
//
// Initialise transmit variables, transmit CAN frame containing total no.
// of frames to receiver & then start the CAN transmit thread running to
// transmit the remaining frames.
//
// Returns
//    Nothing
//--------------------------------------------------------------------------
procedure TForm1.Transmit;
var
  Data : array [0..7] of Byte;
begin
  if RunState then
  begin
    // TRANSMIT
    FrameCount := 0;
    TotalFrames := NoOfFrames.Value;
    Edit2.Text := '0';
    Edit2.Color := clWindow;
    ProgressBar1.Min := 0;
    ProgressBar1.Max := TotalFrames;
    ProgressBar1.Position := 0;
    Label7.Caption := 'Transfer in progress...';
    // Transmit 1st frame containing tot. no. of frames
    Data[0] := TotalFrames and $FF;
    Data[1] := (TotalFrames shr 8) and $FF;
    Data[2] := (TotalFrames shr 16) and $FF;
    Data[3] := (TotalFrames shr 24) and $FF;
    // Check bytes
    Data[4] := $55;
    Data[5] := $AA;
    // Put XSum in last 2 bytes
    XSum := 0;
    CalculateCRC(@Data, 6, XSum);
    Data[6] := XSum and $FF;
    Data[7] := (XSum shr 8) and $FF;
    // Transmit CAN frame
    if CANdoTransmit(@CANdoUSB, CANDO_ID_29_BIT, IDS, CANDO_DATA_FRAME, 8, @Data, 0, 0) <> CANDO_SUCCESS then
      StatusBar1.Panels[1].Text := 'Error communicating with CANdo';
    Wait(10);
    // Create & start transmit thread that sends the data frames
    TransmitThread := CANTransmitThread.Create(True);
    TransmitThread.FreeOnTerminate := True;  // Auto. clean up
    TransmitThread.OnTerminate := TransmitThreadTerminating;
    TransmitRunningFlag := True;
    TransmitThread.Start;  // Run thread
  end;
end;
//--------------------------------------------------------------------------
procedure TForm1.TransmitStop;
begin
  if TransmitRunningFlag then
  begin
    // Thread still running, so terminate
    TransmitThread.Terminate;  // Request thread terminates
    WaitForSingleObject(TransmitThread.Handle, 100);  // Wait for thread to terminate
  end;
end;
//--------------------------------------------------------------------------
procedure TForm1.TransmitThreadTerminating(Sender: TObject);
begin
  // Transmit thread terminating as transmission complete
  if CANdoSetState(@CANdoUSB, CANDO_STOP) = CANDO_SUCCESS then
    Form1.RunState := False;  // Stopped
  TransmitRunningFlag := False;
  RadioButton3.Enabled := True;
  RadioButton4.Enabled := True;
  Button1.Enabled := True;
  Button2.Enabled := False;
  Edit2.Text := IntToStr(XSum);
  if FrameCount = TotalFrames then
  begin
    Label7.Caption := 'Transfer complete - '+IntToStr(FrameCount)+' frames transmitted';
    Edit2.Color := clLime;
  end
  else
  begin
    Label7.Caption := 'Transfer stopped - '+IntToStr(FrameCount)+' frames transmitted';
    Edit2.Color := clYellow;
  end;
end;
//--------------------------------------------------------------------------
// Receive
//
// Initialise receive variables & start Timer1 to collect the CAN frames.
//
// Returns
//    Nothing
//--------------------------------------------------------------------------
procedure TForm1.Receive;
begin
  if RunState then
  begin
    // RECEIVE
    // Clear the CAN buffer
    CANdoCANBuffer.WriteIndex := 0;
    CANdoCANBuffer.ReadIndex := 0;
    CANdoCANBuffer.FullFlag := False;
    // Reset everything before starting
    FrameCount := 0;
    TotalFrames := 0;
    XSum := 0;
    Edit2.Text := '0';
    Edit2.Color := clWindow;
    ProgressBar1.Min := 0;
    ProgressBar1.Max := 0;
    ProgressBar1.Position := 0;
    StartTime := Time;
    Timer1.Interval := 10;
    Timer1.Enabled := True;
    Label7.Caption := 'Receiving...';
  end;
end;
//--------------------------------------------------------------------------
end.
//--------------------------------------------------------------------------
//--------------------------------------------------------------------------
