﻿//------------------------------------------------------------------------------
//  TITLE :- CANdo Interface control for CANdo OBD demo. - CANdo.cs
//  AUTHOR :- Martyn Brown
//  DATE :- 21/10/13
//
//  DESCRIPTION :- CANdo OBD 'C#' demo. program CANdo control.
//
//  UPDATES :-
//	12/01/12 Created
//  21/10/13 1) Updated with CANdo API v4.0
//           2) Modified for 32/64 bit compatibility 
//
//  LICENSE :-
//  This demonstration software is provided for use with the CANdo device &
//  is issued as FREE software, meaning that it is free for personal,
//  educational & commercial use, without restriction or time limit. The
//  software maybe modified & derivative works created, provided that support
//  for the CANdo device is maintained within the software. The software is
//  supplied "as is", with no implied warranties or guarantees.
//
//  (c) 2012-13 Netronics Ltd. All rights reserved.
//------------------------------------------------------------------------------
using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;
using System.Runtime.InteropServices;
using CANdoCSharp;

namespace CANdoOBD
{
  public class CANdo
  {
    //-----------------------------------------------------------------------
    // Accessors.
    //-----------------------------------------------------------------------
    public byte GetRunState { get { return RunState; } }  // CANdo run state
    public string GetCANdoDescription { get { return CANdoDescription; } }  // Description of connected CANdo

    private byte RunState = CANdoImport.STOP;
    private string CANdoDescription = String.Empty;
    private CANdoImport.TCANdoUSB CANdoUSB;  // Store for parameters relating to connected CANdo
    private CANdoImport.TCANdoStatus CANdoStatus;  // Store for status message collected from CANdo
    private Type CANDataType, CANBufferControlType;  // CAN message & buffer types
    private int CANDataSize, CANBufferControlSize, CANBufferSize;  // CAN message & buffer sizes
    private IntPtr CANBufferPointer, CANDataPointer, CANBufferControlPointer;  // Pointers to unmanaged memory
    private CANdoImport.TCANdoCAN CANData;  // CAN message store
    private CANdoImport.TCANdoCAN[] CANDataBuffer;  // Cyclic buffer store for CAN receive messages
    private CANdoImport.TCANdoCANBufferControl CANBufferControl;  // Cyclic buffer control

    //-----------------------------------------------------------------------
    // Init. CANdo variables & message buffers.
    //-----------------------------------------------------------------------
    public CANdo()
    {
      // Create a store for the USB parameters
      CANdoUSB = new CANdoImport.TCANdoUSB();
      CANdoUSB.OpenFlag = CANdoImport.CLOSED;

      // Create a store for the CAN receive messages
      CANDataBuffer = new CANdoImport.TCANdoCAN[CANdoImport.CANDO_CAN_BUFFER_LENGTH];  // Cyclic buffer data
      CANBufferControl = new CANdoImport.TCANdoCANBufferControl();  // Cyclic buffer control
      // Allocate unmanaged memory for the CAN receive message cyclic buffer, including buffer control variables
      CANDataType = typeof(CANdoImport.TCANdoCAN);  // CAN message type
      CANDataSize = Marshal.SizeOf(CANDataType);  // Size of CAN message
      CANBufferControlType = typeof(CANdoImport.TCANdoCANBufferControl);  // CAN buffer control type
      CANBufferControlSize = Marshal.SizeOf(CANBufferControlType);  // Size of CAN buffer control
      CANBufferSize = CANDataBuffer.Length * CANDataSize + CANBufferControlSize;  // Total size in unmanaged memory
      CANBufferPointer = Marshal.AllocHGlobal(CANBufferSize);  // Get pointer to start of CAN buffer in unmanaged memory
      CANBufferControlPointer = CANBufferPointer + CANDataBuffer.Length * CANDataSize;  // Get pointer to control parameters at end of allocated block of memory for CAN receive message cyclic buffer
      // Reset cyclic buffer
      CANBufferControl.WriteIndex = 0;
      CANBufferControl.ReadIndex = 0;
      CANBufferControl.FullFlag = 0;
      // Update unmanaged memory
      Marshal.StructureToPtr(CANBufferControl, CANBufferControlPointer, true);

      // Create a store for the CANdo status message
      CANdoStatus = new CANdoImport.TCANdoStatus();
    }

    //-----------------------------------------------------------------------
    // Clean up.
    //-----------------------------------------------------------------------
    ~CANdo()
    {
      // Free the unmanaged memory
      Marshal.FreeHGlobal(CANBufferPointer);
      CANBufferPointer = IntPtr.Zero;
    }

    //-----------------------------------------------------------------------
    // Open a connection to the first available CANdo.
    //
    // Returns -
    //  false = CANdo not connected
    //  true = CANdo connected
    //-----------------------------------------------------------------------
    public bool Connect()
    {
      bool CANdoConnected = false;

      // Load CANdo.dll & connect to CANdo
      if (CANdoUSB.OpenFlag == CANdoImport.CLOSED)
      {
        try
        {
          if (CANdoImport.CANdoOpen(CANdoUSB) == CANdoImport.CANDO_SUCCESS)
          {
            CANdoDescription = CANdoUSB.Description + " S/N " + CANdoUSB.SerialNo.ToString();
            CANdoConnected = true;
          }
          else
          {
            CANdoImport.CANdoClose(CANdoUSB);  // Unload CANdo.dll
            CANdoDescription = "Netronics CANdo not found";
          }
        }
        catch (DllNotFoundException)
        {
          CANdoDescription = "CANdo.dll file not found";
        }
      }
      return CANdoConnected;
    }

    //-----------------------------------------------------------------------
    // Disconnect the connection to CANdo.
    //
    // Returns -
    //  Nothing
    //-----------------------------------------------------------------------
    public void Disconnect()
    {
      // Close conection to CANdo if open
      if (CANdoUSB.OpenFlag == CANdoImport.OPEN)
      {
        SetState(false);  // Stop CANdo
        CANdoImport.CANdoClose(CANdoUSB);  // Close connection
      }
    }

    //-----------------------------------------------------------------------
    // Start & enable CANdo on the CAN bus at the specified baud rate.
    //
    // Returns -
    //  false = CANdo error
    //  true = CANdo started
    //-----------------------------------------------------------------------
    public bool Start(byte IDE, uint BaudRate)
    {
      if (CANdoUSB.OpenFlag == CANdoImport.OPEN)
      {
        // Init. & start CANdo
        if (SetBaudRate(BaudRate) && SetMode() && SetFilters(IDE) && SetState(true))
          return true;
        else
          return false;
      }
      else
        return false;
    }

    //-----------------------------------------------------------------------
    // Set the CAN baud rate to 250 or 500k baud.
    //
    // Returns -
    //  false = Error setting baud rate
    //  true = Baud rate set
    //-----------------------------------------------------------------------
    public bool SetBaudRate(uint BaudRate)
    {
      if (BaudRate == 250)
      {
        // 250k baud
        if (CANdoImport.CANdoSetBaudRate(CANdoUSB, 0, 1, 7, 5, 4, 0) != CANdoImport.CANDO_SUCCESS)
          return false;  // Error
      }
      else
      {
        // 500k baud, default
        if (CANdoImport.CANdoSetBaudRate(CANdoUSB, 0, 0, 7, 5, 4, 0) != CANdoImport.CANDO_SUCCESS)
          return false;  // Error
      }
      Thread.Sleep(100);  // Wait 100ms to allow CANdo to store settings in EEPROM if changed
      return true;  // Baud rate set
    }

    //-----------------------------------------------------------------------
    // Set the operating mode to 'Normal'.
    //
    // Returns -
    //  false = Error setting mode
    //  true = Mode set
    //-----------------------------------------------------------------------
    public bool SetMode()
    {
      // Set CANdo operating mode
      if (CANdoImport.CANdoSetMode(CANdoUSB, CANdoImport.NORMAL_MODE) == CANdoImport.CANDO_SUCCESS)
      {
        Thread.Sleep(10);  // Wait 10ms to allow CANdo to store setting in EEPROM if changed
        return true;  // Mode set
      }
      else
        return false;  // Error
    }

    //-----------------------------------------------------------------------
    // Set the receive filters to accept either 11 or 29 bit IDs.
    //
    // Returns -
    //  false = Error setting filters
    //  true = Filters set
    //-----------------------------------------------------------------------
    public bool SetFilters(Byte IDE)
    {
      // Set CANdo receive buffer filters
      byte RxIDE;

      if (IDE == CANdoImport.ID_11_BIT)
        RxIDE = CANdoImport.ID_11_BIT;
      else
        RxIDE = CANdoImport.ID_29_BIT;

      // Set receive buffers for either 11 or 29 bit IDs only
      if (CANdoImport.CANdoSetFilters(CANdoUSB,
        0,
        RxIDE, 0,
        RxIDE, 0,
        0,
        RxIDE, 0,
        RxIDE, 0,
        RxIDE, 0,
        RxIDE, 0) == CANdoImport.CANDO_SUCCESS)
      {
        Thread.Sleep(10);  // Wait 10ms to allow filters to be configured in CAN module
        return true;  // Flters set
      }
      else
        return false;  // Error
    }

    //-----------------------------------------------------------------------
    // Set the run state that enables/disables the CAN transceiver & starts/
    // stops the CANdo internal message reception & transmission.
    //
    // Returns -
    //  false = Error setting run state
    //  true = Run state set
    //-----------------------------------------------------------------------
    public bool SetState(bool Run)
    {
      // Set CANdo run state
      if (Run)
        RunState = CANdoImport.RUN;
      else
        RunState = CANdoImport.STOP;

      if ((CANdoImport.CANdoFlushBuffers(CANdoUSB) == CANdoImport.CANDO_SUCCESS) &&
        (CANdoImport.CANdoSetState(CANdoUSB, RunState) == CANdoImport.CANDO_SUCCESS))
        return true;  // State set
      else
        return false;  // Error
    }

    //-----------------------------------------------------------------------
    // Request the CANdo status.
    //
    // Returns -
    //  false = Error requesting status
    //  true = Status requested
    //-----------------------------------------------------------------------
    public bool RequestStatus()
    {
      if (CANdoImport.CANdoRequestStatus(CANdoUSB) == CANdoImport.CANDO_SUCCESS)
        return true;
      else
        return false;
    }

    //-----------------------------------------------------------------------
    // Transmit a CAN message.
    //
    // Returns -
    //  false = message not sent
    //  true = message sent
    //-----------------------------------------------------------------------
    public bool Transmit(byte IDE, uint ID, byte[] Data)
    {
      if (CANdoImport.CANdoTransmit(CANdoUSB, IDE, ID, CANdoImport.DATA_FRAME, 8, Data, 0, 0) == CANdoImport.CANDO_SUCCESS)
        return true;
      else
        return false;
    }

    //-----------------------------------------------------------------------
    // Collect CAN messages received by CANdo.
    //
    // Returns -
    //  false = No status message received
    //  true = Status message received
    //-----------------------------------------------------------------------
    public bool Receive(Main.TCANMessage[] Messages, ref int NoOfMessages, out byte DeviceStatus, out byte CANBusStatus)
    {
      int MessageNo = 0;
      bool StatusReceivedFlag = false;

      DeviceStatus = 0;
      CANBusStatus = 0;
      // Check for CAN messages received
      if (CANdoImport.CANdoReceive(CANdoUSB, CANBufferPointer, CANdoStatus) == CANdoImport.CANDO_SUCCESS)
      {
        // Get the TCANdoCANBufferControl structure populated by the CANdo.dll CANdoReceive function
        CANBufferControl = (CANdoImport.TCANdoCANBufferControl)Marshal.PtrToStructure(CANBufferControlPointer, CANBufferControlType);
        while (((CANBufferControl.ReadIndex != CANBufferControl.WriteIndex) || (CANBufferControl.FullFlag == CANdoImport.SET)) && (MessageNo < NoOfMessages))
        {
          // Get the TCANdoCAN structure for this CAN message populated by the CANdo.dll CANdoReceive function
          CANDataPointer = (IntPtr)(CANBufferPointer + CANBufferControl.ReadIndex * CANDataSize);
          CANData = (CANdoImport.TCANdoCAN)Marshal.PtrToStructure(CANDataPointer, CANDataType);

          // Copy received message to message buffer
          if ((CANData.RTR == CANdoImport.DATA_FRAME) && (CANData.DLC == 8))
          {
            Messages[MessageNo].ID = CANData.ID;
            Messages[MessageNo].IDE = CANData.IDE;
            Array.Copy(CANData.Data, Messages[MessageNo].Data, 8);
            MessageNo++;
          }
          
          // Move read pointer onto next slot in cyclic buffer
          if ((CANBufferControl.ReadIndex + 1) < CANdoImport.CANDO_CAN_BUFFER_LENGTH)
              CANBufferControl.ReadIndex++;  // Increment index onto next free slot
          else
              CANBufferControl.ReadIndex = 0;  // Wrap back to start

          CANBufferControl.FullFlag = CANdoImport.CLEAR;  // Clear flag as buffer not full

          // Update unmanaged memory
          Marshal.StructureToPtr(CANBufferControl, CANBufferControlPointer, true);
        }

        if (CANdoStatus.NewFlag == CANdoImport.CANDO_DEVICE_STATUS)
        {
          // New status message received
          DeviceStatus = CANdoStatus.Status;
          CANBusStatus = CANdoStatus.BusState;
          StatusReceivedFlag = true;
          CANdoStatus.NewFlag = CANdoImport.CANDO_NO_STATUS;  // Clear flag to indicate message read
        }
      }
      NoOfMessages = MessageNo;  // Return the no. of messages received
      return StatusReceivedFlag;
    }
  }
}
//--------------------------------------------------------------------------
//--------------------------------------------------------------------------

