﻿//------------------------------------------------------------------------------
//  TITLE :- Main form for CANdo OBD demo. - Main.cs
//  AUTHOR :- Martyn Brown
//  DATE :- 21/10/13
//
//  DESCRIPTION :- CANdo OBD 'C#' demo. program to communicate with vehicle
//  OBD (On Board Diagnostics) via CAN bus, using ISO 15765 & 15031 protocols.
//
//  UPDATES :-
//	10/01/12 Created
//  21/10/13 Updated with CANdo API v4.0
//
//  LICENSE :-
//  This demonstration software is provided for use with the CANdo device &
//  is issued as FREE software, meaning that it is free for personal,
//  educational & commercial use, without restriction or time limit. The
//  software maybe modified & derivative works created, provided that support
//  for the CANdo device is maintained within the software. The software is
//  supplied "as is", with no implied warranties or guarantees.
//
//  (c) 2012-13 Netronics Ltd. All rights reserved.
//------------------------------------------------------------------------------
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.IO;
using System.Drawing;
using System.Drawing.Imaging;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using CANdoCSharp;

namespace CANdoOBD
{
  public partial class Main : Form
  {
    // Program services
    public const int CURRENT_DATA = 0;  // Current Data service no.
    public const int FREEZE_FRAME = 1;  // Freeze Frame service no.
    public const int FAULT_CODES = 2;  // Fault Code service no.
    public const int CLEAR_FAULTS = 3;  // Clear Faults service no.
    public const int VEHICLE_INFO = 4;  // Vehicle Info. service no.
    public const int FLOW_CONTROL = 5;  // Flow Control Frame service no.
    // Variable store sizes
    public const int CAN_RX_MESSAGE_STORE_SIZE = 20;  // Size of raw CAN frame store for holding OBD reponse frames
    public const int OBD_VALUE_STORE_SIZE = 20;  // Size of store to hold decoded OBD response params.
    // Request timeout & message rates
    public const int CAN_MESSAGE_PROCESSING_PERIOD = 10;  // timer 1 interval in ms
    public const int TX_RESPONSE_TIMEOUT = 3;  // n x timer1 interval timeout on response to OBD request
    public const int SLOW_UPDATE_RATE = 16;  // n x timer1 interval delay between param. requests
    public const int NORMAL_UPDATE_RATE = 4;  // ,,
    public const int FAST_UPDATE_RATE = 1;  // ,,

    // Class to hold a raw CAN message
    public class TCANMessage
    {
      public byte IDE;
      public uint ID;
      public byte[] Data;

      public TCANMessage()
      {
        Data = new byte[8];
      }
    };

    private Settings Settings1;
    private CANdo CANdo1;
    private OBD OBD1;
    private bool CANdoConnected = false, ClearFaultsEnabledFlag = false;
    private int ServiceNo;
    private TCANMessage[] CANRxMessages;
    private byte[] OBDData;
    private bool TxEnabledFlag, TxFlowControlFrameFlag;
    private int TxParamNo, TxTimer, TxDelayPeriod;
        
    public Main()
    {
      InitializeComponent();

      timer1.Interval = CAN_MESSAGE_PROCESSING_PERIOD;

      // Assign list of bitmaps for multi-role buttons
      toolStrip1.ImageList = imageList1;

      // Create an instance of the Settings class & load defaults
      Settings1 = new Settings();

      // Create an instance of the CANdo class
      CANdo1 = new CANdo();

      // Create an instance of the OBD class
      OBD1 = new OBD();

      // Create a store for the CAN messages received from CANdo
      CANRxMessages = new TCANMessage[CAN_RX_MESSAGE_STORE_SIZE];
      for (int No = 0; No < CAN_RX_MESSAGE_STORE_SIZE; No++)
        CANRxMessages[No] = new TCANMessage();

      // Create a store for the OBD request data
      OBDData = new byte[8];
    }

    private void toolStripButton6_Click(object sender, EventArgs e)
    {
      // 'Settings' dialog
      Settings1.ShowDialog();
    }

    private void toolStripButton7_Click(object sender, EventArgs e)
    {
      // 'About' box
      About AboutForm = new About();
      AboutForm.ShowDialog();
    }

    private void toolStripButton8_Click(object sender, EventArgs e)
    {
      // Exit program
      Close();
    }

    private void Main_FormClosing(object sender, FormClosingEventArgs e)
    {
      // Close USB connection to CANdo
      CANdo1.Disconnect();
    }

    private void toolStripButton1_Click(object sender, EventArgs e)
    {
      // 'Connect'/'Disconnect'
      // Uncheck all buttons
      toolStripButton2.Checked = false;
      toolStripButton3.Checked = false;
      toolStripButton4.Checked = false;
      toolStripButton5.Checked = false;
      EnableClearFaultsButton(false);
      if (!CANdoConnected)
      {
        // Connect & start CANdo
        if (CANdo1.Connect())
        {
          CANdoConnected = true;
          toolStripButton1.ImageIndex = 1;
          toolStripButton1.Text = "Disconnect";
          toolStripButton1.ToolTipText = "Disconnect";
          toolStripButton2.Enabled = true;
          toolStripButton3.Enabled = true;
          toolStripButton4.Enabled = true;
          toolStripButton5.Enabled = true;
          toolStripButton6.Enabled = false;
          CANdo1.Start(Settings1.GetIDE, Settings1.GetBaudRate);  // Start CANdo
          Device.Text = CANdo1.GetCANdoDescription + " Connected";
          CANdo1.RequestStatus();  // Request CANdo status
        }
        else
          Device.Text = "Netronics CANdo not found";
      }
      else
      {
        // Stop & disconnect CANdo
        timer1.Enabled = false;
        CANdoConnected = false;
        toolStripButton1.ImageIndex = 0;
        toolStripButton1.Text = "Connect";
        toolStripButton1.ToolTipText = "Connect";
        toolStripButton2.Enabled = false;
        toolStripButton3.Enabled = false;
        toolStripButton4.Enabled = false;
        toolStripButton5.Enabled = false;
        toolStripButton6.Enabled = true;
        listView1.Columns.Clear();
        listView1.Items.Clear();
        CANdo1.Disconnect();
        Device.Text = "";
        CANdoStatus.Text = "";
        BusStatus.Text = "";
      }
    }

    private void toolStripButton2_Click(object sender, EventArgs e)
    {
      // 'Current Data'
      if (!toolStripButton2.Checked)
      {
        // Add column titles
        listView1.Columns.Clear();
        listView1.Columns.Add("Parameter");
        listView1.Columns[0].Width = 330;
        listView1.Columns.Add("Value");
        listView1.Columns[1].Width = 180;
        listView1.Columns.Add("Units");
        listView1.Columns[2].AutoResize(ColumnHeaderAutoResizeStyle.HeaderSize);
        EnableClearFaultsButton(false);
      }
      // Check the selected button
      toolStripButton2.Checked = true;
      toolStripButton3.Checked = false;
      toolStripButton4.Checked = false;
      toolStripButton5.Checked = false;
      ServiceNo = CURRENT_DATA;
      // Display the parameter names & units
      listView1.Items.Clear();
      DisplayParameterNames(ServiceNo, 0, true);
      TxParamNo = 0;  // Start with 1st param. in list
      TxTimer = 0;  // Reset ready to send 1st request
      SetTxDelayPeriod();  // Set delay period between param. requests
      TxEnabledFlag = true;
      timer1.Enabled = true;
    }

    private void toolStripButton3_Click(object sender, EventArgs e)
    {
      // 'Freeze Frame'
      if (!toolStripButton3.Checked)
      {
        // Add column titles
        listView1.Columns.Clear();
        listView1.Columns.Add("Parameter");
        listView1.Columns[0].Width = 330;
        listView1.Columns.Add("Value");
        listView1.Columns[1].Width = 180;
        listView1.Columns.Add("Units");
        listView1.Columns[2].AutoResize(ColumnHeaderAutoResizeStyle.HeaderSize);
        EnableClearFaultsButton(false);
      }
      // Check the selected button
      toolStripButton2.Checked = false;
      toolStripButton3.Checked = true;
      toolStripButton4.Checked = false;
      toolStripButton5.Checked = false;
      ServiceNo = FREEZE_FRAME;
      // Display the 1st parameter name
      listView1.Items.Clear();
      ListViewItem Item = listView1.Items.Add(OBD1.GetParameterName(ServiceNo, 0));
      Item.ForeColor = Color.Orange;
      Item.SubItems.Add("-");
      Item.SubItems.Add(OBD1.GetParameterUnits(ServiceNo, 0));
      TxParamNo = 0;  // Start with 1st param. in list
      TxTimer = 0;  // Reset to indicate ready to send new request
      SetTxDelayPeriod();  // Set delay period between param. requests
      TxEnabledFlag = true;
      timer1.Enabled = true;
    }

    private void toolStripButton4_Click(object sender, EventArgs e)
    {
      // 'Fault Codes'/'Clear Faults'
      if (!ClearFaultsEnabledFlag)
      {
        // 'Fault Codes'
        if (!toolStripButton4.Checked)
        {
          // Add column titles
          listView1.Columns.Clear();
          listView1.Columns.Add("DTC");
          listView1.Columns[0].Width = 120;
          listView1.Columns.Add("Description");
          listView1.Columns[1].AutoResize(ColumnHeaderAutoResizeStyle.HeaderSize);
        }
        // Check the selected button
        toolStripButton2.Checked = false;
        toolStripButton3.Checked = false;
        toolStripButton4.Checked = true;
        toolStripButton5.Checked = false;
        ServiceNo = FAULT_CODES;
        // Display the parameters
        listView1.Items.Clear();
        ListViewItem Item = listView1.Items.Add(OBD1.GetParameterName(ServiceNo, 0));
        Item.ForeColor = Color.Orange;
        Item.SubItems.Add("-");
        TxParamNo = 0;  // Start with 1st param. in list
        TxTimer = 0;  // Reset to indicate ready to send new request
        TxEnabledFlag = true;
        timer1.Enabled = true;
      }
      else
      {
        // 'Clear Faults'
        if (MessageBox.Show("Do you wish to clear all stored faults within the ECU?", "Clear Faults", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
        {
          // Clear all fault codes
          ServiceNo = CLEAR_FAULTS;
          TxParamNo = 0;  // Start with 1st param. in list
          TxTimer = 0;  // Reset to indicate ready to send new request
          TxEnabledFlag = true;
          timer1.Enabled = true;
        }
      }
    }

    private void toolStripButton5_Click(object sender, EventArgs e)
    {
      // 'Vehicle Info.'
      if (!toolStripButton5.Checked)
      {
        // Add column titles
        listView1.Columns.Clear();
        listView1.Columns.Add("Parameter");
        listView1.Columns[0].Width = listView1.Width / 2;
        listView1.Columns.Add("Value");
        listView1.Columns[1].AutoResize(ColumnHeaderAutoResizeStyle.HeaderSize);
        EnableClearFaultsButton(false);
      }
      // Check the selected button
      toolStripButton2.Checked = false;
      toolStripButton3.Checked = false;
      toolStripButton4.Checked = false;
      toolStripButton5.Checked = true;
      ServiceNo = VEHICLE_INFO;
      // Display the parameters
      listView1.Items.Clear();
      DisplayParameterNames(ServiceNo, 0, false);
      TxParamNo = 0;  // Start with 1st param. in list
      TxTimer = 0;  // Reset to indicate ready to send new request
      SetTxDelayPeriod();  // Set delay period between param. requests
      TxEnabledFlag = true;
      timer1.Enabled = true;
    }

    //-----------------------------------------------------------------------
    // Enable 'Clear Faults' button.
    //
    // Returns -
    //  Nothing
    //-----------------------------------------------------------------------
    private void EnableClearFaultsButton(bool State)
    {
      if (State && !ClearFaultsEnabledFlag)
      {
        // Enable 'Clear Faults' button
        toolStripButton4.ImageIndex = 3;
        toolStripButton4.Text = "Clear Faults";
        toolStripButton4.ToolTipText = "Clear Faults";
        ClearFaultsEnabledFlag = true;
      }
      else
      if (!State && ClearFaultsEnabledFlag)
      {
        // Disable 'Clear Faults' button, replace with 'Fault Codes' button
        toolStripButton4.ImageIndex = 2;
        toolStripButton4.Text = "Fault Codes";
        toolStripButton4.ToolTipText = "Fault Codes";
        ClearFaultsEnabledFlag = false;
      }
    }

    //-----------------------------------------------------------------------
    // Display all the parameters in the 'Parameters' list, from the start
    // param. no. to the end of the list.
    //
    // Returns -
    //  Nothing
    //-----------------------------------------------------------------------
    private void DisplayParameterNames(int ServiceNo, int StartParamNo, bool UnitsFlag)
    {
      int ParamNo = StartParamNo;
      int NoOfParams = OBD1.GetNoOfParameters(ServiceNo);
      while (ParamNo < NoOfParams)
      {
        ListViewItem Item = listView1.Items.Add(OBD1.GetParameterName(ServiceNo, ParamNo));
        if ((ParamNo % 2) == 0)
          Item.ForeColor = Color.Blue;
        else
          Item.ForeColor = Color.Green;
        Item.SubItems.Add("-");
        if (UnitsFlag)
          // Display units
          Item.SubItems.Add(OBD1.GetParameterUnits(ServiceNo, ParamNo));
        ParamNo++;
      }
    }

    //-----------------------------------------------------------------------
    // Init. delay period between OBD requests.
    //
    // Returns -
    //  Nothing
    //-----------------------------------------------------------------------
    private void SetTxDelayPeriod()
    {
      if (Settings1.GetUpdateRate == 0)
        TxDelayPeriod = SLOW_UPDATE_RATE;
      else
      if (Settings1.GetUpdateRate == 2)
        TxDelayPeriod = FAST_UPDATE_RATE;
      else
        TxDelayPeriod = NORMAL_UPDATE_RATE;
    }

    //-----------------------------------------------------------------------
    // Receive, decode, display, encode & request OBD parameters every timer1
    // tick.
    //
    // Returns -
    //  Nothing
    //-----------------------------------------------------------------------
    private void timer1_Tick(object sender, EventArgs e)
    {
      int NoOfMessages;
      byte DeviceStatus, CANBusStatus;

      // timer 1 interval elapsed
      NoOfMessages = CANRxMessages.Length;
      // Collect OBD response messages & status from CANdo
      if (CANdo1.Receive(CANRxMessages, ref NoOfMessages, out DeviceStatus, out CANBusStatus))
      {
        // New status message received, so display device & CAN bus status
        if (DeviceStatus == 0)
          CANdoStatus.Text = "Status OK";
        else
          CANdoStatus.Text = "Status " + DeviceStatus.ToString("X2");

        if (CANBusStatus == 0)
          BusStatus.Text = "CAN Bus Status OK";
        else
          BusStatus.Text = "CAN Bus Status " + CANBusStatus.ToString("X2");
      }
      
      // Decode & display received OBD messages
      for (int MessageNo = 0; MessageNo < NoOfMessages; MessageNo++)
      {
        if ((CANRxMessages[MessageNo].IDE == Settings1.GetIDE) && (CANRxMessages[MessageNo].ID == Settings1.GetRxID))
        {
          // CAN IDE & ID match settings, so decode OBD message
          if (OBD1.DecodeMessage(CANRxMessages[MessageNo].Data, ServiceNo))
          {
            // Set flags to trigger transmission of a flow control frame
            TxEnabledFlag = true;
            TxFlowControlFrameFlag = true;
            TxTimer = 0;
          }
          else
          if (OBD1.GetNoOfValues() > 0)
          {
            // One or more values received & decoded, so display
            if (ServiceNo == CLEAR_FAULTS)
              // Response received to clear faults request, so clear all DTCs displayed
              listView1.Items.Clear();

            for (int OBDValueNo = 0; OBDValueNo < OBD1.GetNoOfValues(); OBDValueNo++)
            {
              if (OBD1.GetListNo(OBDValueNo) < listView1.Items.Count)
                // Replace param. value in list
                listView1.Items[OBD1.GetListNo(OBDValueNo)].SubItems[1].Text = OBD1.GetValue(OBDValueNo);
              else
              {
                // Add param. value to end of list
                ListViewItem Item = listView1.Items.Add(OBD1.GetValue(OBDValueNo));
                Item.SubItems.Add(OBD1.GetDTCDescription(OBDValueNo));
                if ((OBDValueNo % 2) == 0)
                  Item.ForeColor = Color.DarkRed;
                else
                  Item.ForeColor = Color.Orange;
              }
              // Check for active DTCs
              if ((ServiceNo == FREEZE_FRAME) && (OBD1.GetListNo(OBDValueNo) == 0) && (OBD1.GetValue(OBDValueNo) != "NONE"))
              {
                // Freeze frame DTC active, so display other freeze frame param. names
                DisplayParameterNames(ServiceNo, 1, true);
                TxEnabledFlag = true;  // Re-enable transmitter to request other freeze frame PIDs
              }
              else
              if ((ServiceNo == FAULT_CODES) && (OBD1.GetListNo(OBDValueNo) == 0) && (OBD1.GetValue(OBDValueNo) == "ON"))
              {
                // MIL on, so enable 'Clear Faults' button, load DTC definitions & request DTCs
                EnableClearFaultsButton(true);  // Enable 'Clear Faults' button
                OBD1.LoadDTCDefintionsFile();  // Load DTC definitions
                TxEnabledFlag = true;  // Re-enable transmitter to request DTCs
              }
            }
            if ((ServiceNo == VEHICLE_INFO) && (TxParamNo < OBD1.GetNoOfParameters(ServiceNo)))
              // Re-enable transmitter to request other vehicle info. params.
              TxEnabledFlag = true;
            TxTimer = TxDelayPeriod;  // Load inter-request delay
          }
        }
      }

      // Transmit OBD request message
      if (TxEnabledFlag)
      {
        // Transmitter enabled
        if (TxTimer == 0)
        {
          // Inter-request time elapsed, so transmit next message
          if (TxFlowControlFrameFlag)
          {
            // Transmit flow control frame
            if (OBD1.EncodeMessage(FLOW_CONTROL, 0, OBDData))
              // Transmit flow control frame using ECU physical ID
              CANdo1.Transmit(Settings1.GetIDE, Settings1.GetTxPhysicalID, OBDData);
            TxEnabledFlag = false;
            TxFlowControlFrameFlag = false;
            TxTimer = TX_RESPONSE_TIMEOUT;
          }
          else
          if (OBD1.EncodeMessage(ServiceNo, TxParamNo, OBDData))
          {
            // Request encoded, so transmit via CANdo
            if (CANdo1.Transmit(Settings1.GetIDE, Settings1.GetTxFunctionalID, OBDData))
            {
              // Move onto next param.
              TxParamNo++;
              if (ServiceNo == CURRENT_DATA)
              {
                if (TxParamNo == OBD1.GetNoOfParameters(ServiceNo))
                  TxParamNo = 0;  // Go back to start of list
              }
              else
              if ((ServiceNo == FREEZE_FRAME) || (ServiceNo == FAULT_CODES))
              {
                if ((TxParamNo == 1) || (TxParamNo == OBD1.GetNoOfParameters(ServiceNo)))
                  TxEnabledFlag = false;  // Disable transmitter if the 1st or last param. in list
              }
              else
              if (ServiceNo == FAULT_CODES)
                TxEnabledFlag = false;  // Disable transmitter
              else
              if (ServiceNo == CLEAR_FAULTS)
              {
                EnableClearFaultsButton(false);  // Change 'Clear Faults' button back to 'Fault Codes'
                toolStripButton4.Checked = false;  // Uncheck button
                TxEnabledFlag = false;  // Disable transmitter
              }
              else
              if (ServiceNo == VEHICLE_INFO)
              {
                if (TxParamNo == OBD1.GetNoOfParameters(ServiceNo))
                  TxEnabledFlag = false;  // Disable transmitter as param. list complete
              }
              // Load response timeout
              TxTimer = TX_RESPONSE_TIMEOUT;
            }
          }
        }
        else
        if (TxTimer > 0)
          TxTimer--;  // Decrement timeout/inter-request delay timer
      }
    }
  }
}
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
