﻿//------------------------------------------------------------------------------
//  TITLE :- OBD message encode/decode for CANdo OBD demo. - OBD.cs
//  AUTHOR :- Martyn Brown
//  DATE :- 18/01/12
//
//  DESCRIPTION :- CANdo OBD 'C#' demo. program OBD message encode & decode in
//  accordance with the ISO 15031-5 protocol (SAE J1979).
//
//  UPDATES :-
//	18/01/12 Created
//
//  LICENSE :-
//  This demonstration software is provided for use with the CANdo device &
//  is issued as FREE software, meaning that it is free for personal,
//  educational & commercial use, without restriction or time limit. The
//  software maybe modified & derivative works created, provided that support
//  for the CANdo device is maintained within the software. The software is
//  supplied "as is", with no implied warranties or guarantees.
//
//  (c) 2012 Netronics Ltd. All rights reserved.
//------------------------------------------------------------------------------
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace CANdoOBD
{
  public class OBD
  {
    // Buffer sizes
    public const int MAX_NO_OF_VALUES = 20;  // Max. no. of extracted values
    public const int MAX_MULTI_FRAME_MESSAGE_SIZE = 4095;  // Max. no. of bytes in a multi-frame CAN message
    // OBD CAN frame types
    public const byte SINGLE_FRAME = 0;  // Single frame message
    public const byte FIRST_FRAME = 1;  // First frame of a multi-frame message
    public const byte CONSECUTIVE_FRAME = 2;  // Consecutive frame of a multi-frame message
    public const byte FLOW_FRAME = 3;  // Flow frame for a multi-frame message
    // OBD constants
    public const byte PADDING = 0x55;  // Data padding for request frame
    public const byte SID_RESPONSE_FLAG = 0x40;  // Flag bit set in SID response byte
    public const byte NO_OF_BYTES_IN_FIRST_FRAME = 6;  // No. of data bytes in the 1st frame of a multi-frame message
    public const byte NO_OF_BYTES_IN_CONSECUTIVE_FRAME = 7;  // No. of data bytes in a consecutive frame of a multi-frame message
    // OBD fault code definitions file
    public const string DTCFileName = "DTCs.txt";

    // OBD value types
    public enum TValueType { INT_VALUE, FUEL_SYSTEM_VALUE, FREEZE_FRAME_DTC_VALUE, MIL_VALUE, DTC_VALUE, CLEAR_DTC_VALUE, VEHICLE_INFO_VALUE };

    // Structure type for an OBD param.
    private struct TParameter
    {
      public string Name;
      public string Units;
      public byte PCI;
      public byte SID;
      public byte PID;
      public byte FrameNo;
      public float Offset;
      public float ScalingFactor;
      public TValueType ValueType;
    }

    // List of OBD params. as per ISO 15031-5 (SAE J1979)
    private readonly TParameter[][] Parameters =
    {
      // 'Current Data' parameters
      new TParameter[]
      {
        new TParameter() { Name = "Vehicle Speed", Units = "km/h", PCI = 0x02, SID = 0x01, PID = 0x0D, Offset = 0.0f, ScalingFactor = 1.0f, ValueType = TValueType.INT_VALUE },
        new TParameter() { Name = "Engine Speed", Units = "rpm", PCI = 0x02, SID = 0x01, PID = 0x0C, Offset = 0.0f, ScalingFactor = 0.25f, ValueType = TValueType.INT_VALUE },
        new TParameter() { Name = "Engine Coolant Temperature", Units = "\u00B0C", PCI = 0x02, SID = 0x01, PID = 0x05, Offset = -40.0f, ScalingFactor = 1.0f, ValueType = TValueType.INT_VALUE },
        new TParameter() { Name = "Absolute Throttle Position", Units = "%", PCI = 0x02, SID = 0x01, PID = 0x05, Offset = 0.0f, ScalingFactor = 1.0f, ValueType = TValueType.INT_VALUE },
        new TParameter() { Name = "Mass Air Flow Rate", Units = "Grams/s", PCI = 0x02, SID = 0x01, PID = 0x10, Offset = 0.0f, ScalingFactor = 0.01f, ValueType = TValueType.INT_VALUE },
        new TParameter() { Name = "Calculated Engine Load", Units = "%", PCI = 0x02, SID = 0x01, PID = 0x04, Offset = 0.0f, ScalingFactor = 0.39216f, ValueType = TValueType.INT_VALUE },
        new TParameter() { Name = "Intake Air Temperature", Units = "\u00B0C", PCI = 0x02, SID = 0x01, PID = 0x0F, Offset = -40.0f, ScalingFactor = 1.0f, ValueType = TValueType.INT_VALUE },
        new TParameter() { Name = "Fuel Level Input", Units = "%", PCI = 0x02, SID = 0x01, PID = 0x2F, Offset = 0.0f, ScalingFactor = 0.39216f, ValueType = TValueType.INT_VALUE },
        new TParameter() { Name = "Fuel Rail Pressure", Units = "kPa", PCI = 0x02, SID = 0x01, PID = 0x0A, Offset = 0.0f, ScalingFactor = 3.0f, ValueType = TValueType.INT_VALUE },
        new TParameter() { Name = "Fuel System Status", Units = "-", PCI = 0x02, SID = 0x01, PID = 0x03, ValueType = TValueType.FUEL_SYSTEM_VALUE },
        new TParameter() { Name = "Barometric Pressure", Units = "kPa", PCI = 0x02, SID = 0x01, PID = 0x33, Offset = 0.0f, ScalingFactor = 1.0f, ValueType = TValueType.INT_VALUE },
        new TParameter() { Name = "Control Module Voltage", Units = "V", PCI = 0x02, SID = 0x01, PID = 0x42, Offset = 0.0f, ScalingFactor = 0.001f, ValueType = TValueType.INT_VALUE }
      },
      // 'Freeze Frame' parameters
      new TParameter[]
      {
        new TParameter() { Name = "Freeze Frame DTC", Units = "", PCI = 0x03, SID = 0x02, PID = 0x02, FrameNo = 0, ValueType = TValueType.FREEZE_FRAME_DTC_VALUE },
        new TParameter() { Name = "Vehicle Speed", Units = "km/h", PCI = 0x03, SID = 0x02, PID = 0x0D, Offset = 0.0f, ScalingFactor = 1.0f, ValueType = TValueType.INT_VALUE },
        new TParameter() { Name = "Engine Speed", Units = "rpm", PCI = 0x03, SID = 0x02, PID = 0x0C, Offset = 0.0f, ScalingFactor = 0.25f, ValueType = TValueType.INT_VALUE },
        new TParameter() { Name = "Engine Coolant Temperature", Units = "\u00B0C", PCI = 0x03, SID = 0x02, PID = 0x05, Offset = -40.0f, ScalingFactor = 1.0f, ValueType = TValueType.INT_VALUE },
        new TParameter() { Name = "Absolute Throttle Position", Units = "%", PCI = 0x03, SID = 0x02, PID = 0x05, Offset = 0.0f, ScalingFactor = 1.0f, ValueType = TValueType.INT_VALUE },
        new TParameter() { Name = "Fuel Rail Pressure", Units = "kPa", PCI = 0x03, SID = 0x02, PID = 0x0A, Offset = 0.0f, ScalingFactor = 3.0f, ValueType = TValueType.INT_VALUE }
      },
      // 'Fault Codes' parameters
      new TParameter[]
      {
        new TParameter() { Name = "MIL Status", PCI = 0x02, SID = 0x01, PID = 0x01, ValueType = TValueType.MIL_VALUE },
        new TParameter() { PCI = 0x01, SID = 0x03, ValueType = TValueType.DTC_VALUE }
      },
      // 'Clear Faults' parameters
      new TParameter[]
      {
        new TParameter() { PCI = 0x01, SID = 0x04, ValueType = TValueType.CLEAR_DTC_VALUE }
      },
      // 'Vehicle Info.' parameters
      new TParameter[]
      {
        new TParameter() { Name = "Vehicle Identification No.", PCI = 0x02, SID = 0x09, PID = 0x02, ValueType = TValueType.VEHICLE_INFO_VALUE },
        new TParameter() { Name = "Calibration Identification", PCI = 0x02, SID = 0x09, PID = 0x04, ValueType = TValueType.VEHICLE_INFO_VALUE },
        new TParameter() { Name = "ECU Name", PCI = 0x02, SID = 0x09, PID = 0x0A, ValueType = TValueType.VEHICLE_INFO_VALUE }
      },
      // 'Flow Control Frame' parameters
      new TParameter[]
      {
        new TParameter() { PCI = 0x30, SID = 0x0A, PID = 0x02 }  // PCI = Flow control frame, SID = Max. 10 frames between flow control frames & PID = 2ms min. inter-frame spacing
      }
    };

    // Parameters array accessors
    public int GetNoOfParameters(int ServiceNo)
    {
      return Parameters[ServiceNo].Length;
    }

    public string GetParameterName(int ServiceNo, int ParamNo)
    {
      return Parameters[ServiceNo][ParamNo].Name;
    }

    public string GetParameterUnits(int ServiceNo, int ParamNo)
    {
      return Parameters[ServiceNo][ParamNo].Units;
    }

    private struct TOBDValue
    {
      public int ListNo;  // Position for value in display list
      public string Value;  // Value extracted from received message
    }

    // OBD values array accessors
    public int GetNoOfValues()
    {
      return NoOfValues;
    }

    public int GetListNo(int ValueNo)
    {
      return Values[ValueNo].ListNo;
    }

    public string GetValue(int ValueNo)
    {
      return Values[ValueNo].Value;
    }

    // OBD DTC decription array accessor
    public string GetDTCDescription(int DTCNo)
    {
      return DTCDescriptions[DTCNo];
    }

    private byte[] CANDataBuffer;  // Multi-frame CAN data buffer
    private int NoOfAdditionalBytes, ByteNo, NoOfAdditionalFrames, FrameNo;
    private int NoOfValues;
    private TOBDValue[] Values;  // Decoded OBD param. value store
    private bool DTCDefinitionFileLoadedFlag = false;  // Flag to indicate DTC defin. file loaded
    private List<string> DTCDefinitions;  // String store to hold the DTC definitions
    private string[] DTCDescriptions;  // DTC descriptions for the displayed fault codes

    //-----------------------------------------------------------------------
    // Init. OBD variables & buffers.
    //-----------------------------------------------------------------------
    public OBD()
    {
      // Create a store for multi-frame CAN data
      CANDataBuffer = new byte[MAX_MULTI_FRAME_MESSAGE_SIZE];
      // Create a store for the extracted values
      Values = new TOBDValue[MAX_NO_OF_VALUES];
      // Create a store for the DTC descriptions
      DTCDescriptions = new string[MAX_NO_OF_VALUES];
    }

    //-----------------------------------------------------------------------
    // Scan the 'Parameters' list for the specified 'PID' & return with the
    // param. no. in the list.
    //
    // Returns -
    //  false = Param. not found
    //  true = Param. found
    //-----------------------------------------------------------------------
    private bool LookUpPID(int ServiceNo, byte SID, byte PID, out int ParamNo)
    {
      ParamNo = 0;
      while (ParamNo < Parameters[ServiceNo].Length)
      {
        if (SID == (Parameters[ServiceNo][ParamNo].SID | SID_RESPONSE_FLAG))
        {
          // SID matches
          if ((Parameters[ServiceNo][ParamNo].PCI & 0x0F) == 1)
            // No PID for this SID
            return true;  // Param. found
          else
          if (Parameters[ServiceNo][ParamNo].PID == PID)
            // Param. found in list, so return with param. no.
            return true;  // Param. found
        }
        ParamNo++;
      }
      // Param. not found
      return false;
    }

    //-----------------------------------------------------------------------
    // Convert parameter(s) according to type.
    //
    // Returns -
    //  Nothing
    //-----------------------------------------------------------------------
    private void ConvertParam(int ServiceNo, int ParamNo, byte[] CANData, int NoOfAdditionalBytes)
    {
      if (Parameters[ServiceNo][ParamNo].ValueType == TValueType.INT_VALUE)
      {
        // Convert int value
        int Value = 0;
        for (int ByteNo = 0; ByteNo < (NoOfAdditionalBytes - 2); ByteNo++)
          Value += CANData[ByteNo + 1] << ((NoOfAdditionalBytes - 3 - ByteNo) * 8);
        // Add value to Values array
        Values[NoOfValues].ListNo = ParamNo;
        Values[NoOfValues].Value = ((float)Value * Parameters[ServiceNo][ParamNo].ScalingFactor + Parameters[ServiceNo][ParamNo].Offset).ToString("0.0");
        NoOfValues++;
      }
      else
      if (Parameters[ServiceNo][ParamNo].ValueType == TValueType.FUEL_SYSTEM_VALUE)
      {
        // Decode fuel sys. 1 status
        Values[NoOfValues].ListNo = ParamNo;
        switch (CANData[1])
        {
          case 0:
            Values[NoOfValues].Value = "Open Loop";
            break;
          case 1:
            Values[NoOfValues].Value = "Closed Loop";
            break;
          case 2:
            Values[NoOfValues].Value = "Open Loop (Drive)";
            break;
          case 4:
            Values[NoOfValues].Value = "Open Loop (Fault)";
            break;
          case 8:
            Values[NoOfValues].Value = "Closed Loop (Fault)";
            break;
          default:
            Values[NoOfValues].Value = "Unknown";
            break;
        }
        NoOfValues++;
      }
      else
      if (Parameters[ServiceNo][ParamNo].ValueType == TValueType.FREEZE_FRAME_DTC_VALUE)
      {
        // Extract the DTC that caused the freeze frame
        Values[NoOfValues].ListNo = ParamNo;
        Values[NoOfValues].Value = ExtractDTC(CANData[2], CANData[3]);
        NoOfValues++;
      }
      else
      if (Parameters[ServiceNo][ParamNo].ValueType == TValueType.MIL_VALUE)
      {
        // Extract MIL status
        Values[NoOfValues].ListNo = ParamNo;
        if ((CANData[1] & 0x80) == 0)
          Values[NoOfValues].Value = "OFF";  // MIL off
        else
          Values[NoOfValues].Value = "ON";  // MIL on
        NoOfValues++;
      }
      else
      if (Parameters[ServiceNo][ParamNo].ValueType == TValueType.DTC_VALUE)
      {
        // Extract the DTCs
        int DTCNo = 0;
        string DTC;
        while ((DTCNo < CANData[0]) && (NoOfValues < MAX_NO_OF_VALUES))
        {
          // Add each decoded DTC to the array of values
          Values[NoOfValues].ListNo = ParamNo + DTCNo;
          DTC = ExtractDTC(CANData[DTCNo * 2 + 1], CANData[DTCNo * 2 + 2]);
          Values[NoOfValues].Value = DTC;
          DTCDescriptions[NoOfValues] = LookUpDTCDescription(DTC);
          NoOfValues++;
          DTCNo++;
        }
      }
      else
      if (Parameters[ServiceNo][ParamNo].ValueType == TValueType.CLEAR_DTC_VALUE)
      {
        // DTCs cleared
        Values[NoOfValues].ListNo = ParamNo;
        Values[NoOfValues].Value = "CLEARED!";  // DTCs cleared
        DTCDescriptions[NoOfValues] = "";
        NoOfValues++;
      }
      else
      if (Parameters[ServiceNo][ParamNo].ValueType == TValueType.VEHICLE_INFO_VALUE)
      {
        // Extract vehicle info.
        Values[NoOfValues].ListNo = ParamNo;
        Values[NoOfValues].Value = ASCIIEncoding.ASCII.GetString(CANData.ToArray(), 2, NoOfAdditionalBytes - 3);
        NoOfValues++;
      }
    }

    //-----------------------------------------------------------------------
    // Extract the DTC from the 2 bytes of data supplied.
    //
    // Returns -
    //  DTC string value
    //-----------------------------------------------------------------------
    private static string ExtractDTC(byte HighByte, byte LowByte)
    {
      string DTC = String.Empty;
      int LetterCode = (HighByte & 0xC0) >> 6;
      int HexDigit;

      switch (LetterCode)
      {
        case 0:
          DTC = "P";
          break;

        case 1:
          DTC = "C";
          break;

        case 2:
          DTC = "B";
          break;

        case 3:
          DTC = "U";
          break;
      }

      HexDigit = (HighByte & 0x30) >> 4;
      DTC += HexDigit.ToString("X");
      HexDigit = HighByte & 0x0F;
      DTC += HexDigit.ToString("X");
      HexDigit = (LowByte & 0xF0) >> 4;
      DTC += HexDigit.ToString("X");
      HexDigit = LowByte & 0x0F;
      DTC += HexDigit.ToString("X");

      if (DTC == "P0000")
        return "NONE";
      else
        return DTC;
    }

    //-----------------------------------------------------------------------
    // Scan the 'DTCDefinitions' list for the specified 'DTC' & return with
    // the DTC description.
    //
    // Returns -
    //  DTC description
    //-----------------------------------------------------------------------
    private string LookUpDTCDescription(string DTC)
    {
      int DTCNo, NoOfDTCs;

      if (DTCDefinitionFileLoadedFlag)
      {
        DTCNo = 0;
        NoOfDTCs = DTCDefinitions.Count;
        while (DTCNo < NoOfDTCs)
        {
          if (DTCDefinitions[DTCNo].StartsWith(DTC))
            // DTC found, so return description
            return DTCDefinitions[DTCNo].Substring(6);
          DTCNo++;
        }
        return "?";  // Description not found
      }
      else
        return "-";  // No descriptions loaded
    }

    //-----------------------------------------------------------------------
    // Load DTC definitions file into a string list.
    //
    // Returns -
    //  false = DTC file not loaded
    //  true = DTC file loaded
    //-----------------------------------------------------------------------
    public bool LoadDTCDefintionsFile()
    {
      if (!DTCDefinitionFileLoadedFlag && File.Exists(DTCFileName))
      {
        // Init. DTC definition store
        DTCDefinitions = new List<string>();
        // Load definitions from file
        try
        {
          string DTCDefinition;
          using (StreamReader SR = new StreamReader(DTCFileName))
          {
            // Load definitions
            while ((DTCDefinition = SR.ReadLine()) != null)
              // Store in list
              DTCDefinitions.Add(DTCDefinition);
          }
          DTCDefinitionFileLoadedFlag = true;
          return true;
        }
        catch
        {
          // Empty list
          DTCDefinitions.Clear();
          return false;
        }
      }
      else
        return false;
    }

    //-----------------------------------------------------------------------
    // Decode raw CAN frame according to ISO 15765-2 & ISO 15031-5 protocols.
    //
    // Returns -
    //  false = No flow control frame required
    //  true = Flow control frame required
    //-----------------------------------------------------------------------
    public bool DecodeMessage(byte[] CANData, int ServiceNo)
    {
      bool FlowControlFrameFlag = false;
      int FrameType = CANData[0] >> 4;
      int ParamNo;

      // Decode PCI
      if (FrameType == SINGLE_FRAME)
      {
        // Single frame response, so decode CAN data
        NoOfValues = 0;  // Reset values count
        NoOfAdditionalBytes = CANData[0] & 0x0F;  // No. of additional bytes in message
        if (LookUpPID(ServiceNo, CANData[1], CANData[2], out ParamNo))
          // Param. found in Parameters array, so convert data according to param. type
          ConvertParam(ServiceNo, ParamNo, CANData.Skip(2).ToArray(), NoOfAdditionalBytes);
      }
      else
      if (FrameType == FIRST_FRAME)
      {
        // First frame in a multi-frame response
        NoOfValues = 0;  // Reset values count
        NoOfAdditionalBytes = ((CANData[0] & 0x0F) << 8) + CANData[1];  // No. of additional bytes in message
        NoOfAdditionalFrames = NoOfAdditionalBytes / NO_OF_BYTES_IN_CONSECUTIVE_FRAME;  // No. of additional frames in multi-frame message
        FrameNo = 1;
        ByteNo = 0;
        for (int DataNo = 0; DataNo < NO_OF_BYTES_IN_FIRST_FRAME; DataNo++)
          // Copy frame data to multi-frame CAN buffer
          CANDataBuffer[ByteNo++] = CANData[DataNo + 2];
        // Set flag to indicate flow control frame required
        FlowControlFrameFlag = true;
      }
      else
      if (FrameType == CONSECUTIVE_FRAME)
      {
        // Consecutive frame in a multi-frame response
        if ((CANData[0] & 0x0F) == FrameNo)
        {
          // Correct frame in sequence
          for (int DataNo = 0; DataNo < NO_OF_BYTES_IN_CONSECUTIVE_FRAME; DataNo++)
            // Copy frame data to multi-frame CAN buffer
            CANDataBuffer[ByteNo++] = CANData[DataNo + 1];
          // Check for last frame in sequence
          if (FrameNo < NoOfAdditionalFrames)
            FrameNo++;
          else
          {
            // Last frame of multi-frame message stored in buffer, so convert params.
            if (LookUpPID(ServiceNo, CANDataBuffer[0], CANDataBuffer[1], out ParamNo))
              // Param. found in Parameters array, so convert data according to param. type
              ConvertParam(ServiceNo, ParamNo, CANDataBuffer.Skip(1).ToArray(), NoOfAdditionalBytes);
          }
        }
      }
      return FlowControlFrameFlag;
    }

    //-----------------------------------------------------------------------
    // Encode an OBD request message in accordance with the ISO 15031-5
    // protocol.
    //
    // Returns -
    //  false = Message not encoded
    //  true = Message encoded
    //-----------------------------------------------------------------------
    public bool EncodeMessage(int ServiceNo, int ParamNo, byte[] OBDRequestData)
    {
      if ((ServiceNo < Parameters.Length) && (ParamNo < Parameters[ServiceNo].Length))
      {
        // Encode message
        int DataNo = 0;
        OBDRequestData[DataNo++] = Parameters[ServiceNo][ParamNo].PCI;  // PCI
        int NoOfDataBytes = Parameters[ServiceNo][ParamNo].PCI & 0x0F;  // No. of additional data bytes in request
        OBDRequestData[DataNo++] = Parameters[ServiceNo][ParamNo].SID;  // SID
        if (NoOfDataBytes > 1)
          // Load PID from Parameters array
          OBDRequestData[DataNo++] = Parameters[ServiceNo][ParamNo].PID;  // PID
        if (NoOfDataBytes > 2)
          // Load frame no. from Parameters array
          OBDRequestData[DataNo++] = 0;  // Frame no.
        while (DataNo < 8)
          // Padding
          OBDRequestData[DataNo++] = PADDING;
        return true;
      }
      else
        return false;
    }
  }
}
//--------------------------------------------------------------------------
//--------------------------------------------------------------------------

